@@ -516,9 +516,9 @@ class TestGtk(unittest.TestCase):
         widget.drag_source_add_image_targets()
         widget.drag_source_add_text_targets()
         widget.drag_source_add_uri_targets()
-        widget.drag_source_set_icon_name("")
+        widget.drag_source_set_icon_name("_About")
         widget.drag_source_set_icon_pixbuf(GdkPixbuf.Pixbuf())
-        widget.drag_source_set_icon_stock("")
+        widget.drag_source_set_icon_stock(Gtk.STOCK_ABOUT)
         widget.drag_source_get_target_list()
         widget.drag_source_set_target_list(None)
         widget.drag_source_set_target_list(Gtk.TargetList.new([Gtk.TargetEntry.new('test', 0, 0)]))
